@@ -1,4 +1,4 @@
-/*	$NetBSD: chared.c,v 1.62 2022/02/08 21:13:22 rillig Exp $	*/
+/*	$NetBSD: chared.c,v 1.64 2024/06/29 14:13:14 christos Exp $	*/
 
 /*-
  * Copyright (c) 1992, 1993
@@ -37,7 +37,7 @@
 #if 0
 static char sccsid[] = "@(#)chared.c	8.1 (Berkeley) 6/4/93";
 #else
-__RCSID("$NetBSD: chared.c,v 1.62 2022/02/08 21:13:22 rillig Exp $");
+__RCSID("$NetBSD: chared.c,v 1.64 2024/06/29 14:13:14 christos Exp $");
 #endif
 #endif /* not lint && not SCCSID */
 
@@ -319,6 +319,8 @@ cv_prev_word(wchar_t *p, wchar_t *low, int n, int (*wtest)(wint_t))
 		test = (*wtest)(*p);
 		while ((p >= low) && (*wtest)(*p) == test)
 			p--;
+		if (p < low)
+			return low;
 	}
 	p++;
 
@@ -414,7 +416,7 @@ ch_init(EditLine *el)
 	el->el_chared.c_redo.buf	= el_calloc(EL_BUFSIZ,
 	    sizeof(*el->el_chared.c_redo.buf));
 	if (el->el_chared.c_redo.buf == NULL)
-		return -1;
+		goto out;
 	el->el_chared.c_redo.pos	= el->el_chared.c_redo.buf;
 	el->el_chared.c_redo.lim	= el->el_chared.c_redo.buf + EL_BUFSIZ;
 	el->el_chared.c_redo.cmd	= ED_UNASSIGNED;
@@ -425,7 +427,7 @@ ch_init(EditLine *el)
 	el->el_chared.c_kill.buf	= el_calloc(EL_BUFSIZ,
 	    sizeof(*el->el_chared.c_kill.buf));
 	if (el->el_chared.c_kill.buf == NULL)
-		return -1;
+		goto out;
 	el->el_chared.c_kill.mark	= el->el_line.buffer;
 	el->el_chared.c_kill.last	= el->el_chared.c_kill.buf;
 	el->el_chared.c_resizefun	= NULL;
@@ -442,6 +444,9 @@ ch_init(EditLine *el)
 	el->el_state.lastcmd		= ED_UNASSIGNED;
 
 	return 0;
+out:
+	ch_end(el);
+	return -1;
 }
 
 /* ch_reset():
